@@ -62,7 +62,6 @@ int myumask = 0755;
 
 int max_xmit = BUFFER_SIZE;
 
-extern BOOL NeedSwap;
 extern pstring scope;
 
 BOOL prompt = True;
@@ -118,7 +117,7 @@ setup_term_code (char *code)
 #endif
 
 static void send_logout(void );
-
+BOOL reopen_connection(char *inbuf,char *outbuf);
 
 /****************************************************************************
 setup basics in a outgoing packet
@@ -335,6 +334,8 @@ static void send_message(char *inbuf,char *outbuf)
       int l=0;
       int c;
 
+      memset(msg,0,smb_size);
+
       for (l=0;l<maxlen && (c=fgetc(stdin))!=EOF;l++)
 	{
 	  if (c == '\n')
@@ -466,7 +467,7 @@ void dir_action(char *inbuf,char *outbuf,int attribute,file_info *finfo,BOOL rec
 {
 
   if (!((finfo->mode & aDIR) == 0 && *fileselection && 
-	!mask_match(finfo->name,fileselection,False,False,True)) &&
+	!mask_match(finfo->name,fileselection,False,True)) &&
       !(recurse_dir && (strequal(finfo->name,".") || 
 			strequal(finfo->name,".."))))
     {
@@ -540,6 +541,8 @@ int do_short_dir(char *inbuf,char *outbuf,char *Mask,int attribute,void (*fn)(),
 
   finfo = def_finfo;
 
+  memset(status,0,21);
+
   strcpy(mask,Mask);
   
   while (1)
@@ -1172,7 +1175,7 @@ void do_get(char *rname,char *lname,file_info *finfo1)
       /* if we can finish now then readX+close */
       if (method<0 && can_chain_close && (Protocol >= PROTOCOL_LANMAN1) && 
 	  ((finfo.size - nread) < 
-	   (max_xmit - (2*smb_size + 13*sizeof(WORD) + 300))))
+	   (max_xmit - (2*smb_size + 13*SIZEOFWORD + 300))))
 	method = 0;
 
       /* if we support readraw then use that */
@@ -1365,11 +1368,7 @@ void do_get(char *rname,char *lname,file_info *finfo1)
       setup_pkt(outbuf);
       
       SSVAL(outbuf,smb_vwv0,fnum);
-#if 0
-      SIVAL(outbuf,smb_vwv1,finfo.mtime);
-#else
       SIVALS(outbuf,smb_vwv1,-1);
-#endif
       
       send_smb(outbuf);
       receive_smb(inbuf,CLIENT_TIMEOUT,False);
@@ -1547,12 +1546,17 @@ void cmd_mget(char *inbuf,char *outbuf)
 	strcpy(mget_mask,p);
       else
 	strcat(mget_mask,p);
+      do_dir((char *)inbuf,(char *)outbuf,mget_mask,attribute,do_mget,False);
     }
 
   if (! *mget_mask)
-    strcpy(mget_mask,"*.*");
-
-  do_dir((char *)inbuf,(char *)outbuf,mget_mask,attribute,do_mget,False);
+    {
+      strcpy(mget_mask,cur_dir);
+      if(mget_mask[strlen(mget_mask)-1]!='\\')
+	strcat(mget_mask,"\\");
+      strcat(mget_mask,"*.*");
+      do_dir((char *)inbuf,(char *)outbuf,mget_mask,attribute,do_mget,False);
+    }
 }
 
 /****************************************************************************
@@ -2245,24 +2249,14 @@ void cmd_print(char *inbuf,char *outbuf )
 	 packets when printing. weird */
       n = MIN(1024,max_xmit-(smb_len(outbuf)+4));
 
-#if 0
-      if (first)
-	{
-	  n = 0;
-	  first = False;
-	}
+      if (translation)
+	n = printread(f,smb_buf(outbuf)+3,(int)(0.95*n));
       else
-#endif
+	n = readfile(smb_buf(outbuf)+3,1,n,f);
+      if (n <= 0) 
 	{
-	  if (translation)
-	    n = printread(f,smb_buf(outbuf)+3,(int)(0.95*n));
-	  else
-	    n = readfile(smb_buf(outbuf)+3,1,n,f);
-	  if (n <= 0) 
-	    {
-	      DEBUG(0,("read gave %d\n",n));
-	      break;
-	    }
+	  DEBUG(0,("read gave %d\n",n));
+	  break;
 	}
 
       smb_setlen(outbuf,smb_len(outbuf) + n);
@@ -2613,7 +2607,6 @@ static BOOL send_session_request(char *inbuf,char *outbuf)
   char *p;
   int len = 4;
   /* send a session request (RFC 8002) */
-  CVAL(outbuf,0) = 0x81;
 
   /* put in the destination name */
   p = outbuf+len;
@@ -2630,16 +2623,8 @@ static BOOL send_session_request(char *inbuf,char *outbuf)
   len += name_len(p);
 
   /* setup the packet length */
-  /* We can't use smb_setlen here as it assumes a data
-     packet and will trample over the name data we have copied
-     in (by adding 0xFF 'S' 'M' 'B' at offsets 4 - 7 */
-
-  NeedSwap = !NeedSwap;
-  SSVAL(outbuf,2,len);
-  NeedSwap = !NeedSwap;
-
-  if (len >= (1 << 16))
-    CVAL(outbuf,1) |= 1;
+  _smb_setlen(outbuf,len);
+  CVAL(outbuf,0) = 0x81;
 
   send_smb(outbuf);
   DEBUG(5,("Sent session request\n"));
@@ -2874,10 +2859,6 @@ static BOOL send_login(char *inbuf,char *outbuf,BOOL start_session,BOOL use_setu
 
   /* now we've got a connection - send a tcon message */
   memset(outbuf,0,smb_size);
-#if 0
-  if (Protocol >= PROTOCOL_LANMAN1)
-    strcpy(pass,"");
-#endif
 
   if (strncmp(service,"\\\\",2) != 0)
     {
@@ -2967,19 +2948,6 @@ static void send_logout(void )
     {
       DEBUG(0,("SMBtdis failed %s\n",smb_errstr(inbuf)));
     }
-#if 0
-  set_message(outbuf,0,0,True);
-  CVAL(outbuf,smb_com) = SMBexit;
-  setup_pkt(outbuf);
-
-  send_smb(outbuf);
-  receive_smb(inbuf,CLIENT_TIMEOUT,False);
-
-  if (CVAL(inbuf,smb_rcls) != 0)
-    {
-      DEBUG(0,("SMBexit failed %s\n",smb_errstr(inbuf)));
-    }
-#endif
 
   
 #ifdef STATS
@@ -3089,7 +3057,7 @@ static BOOL browse_host()
       int i;
 
       if (res != 0)
-	return;
+	return(False);
 
       p = rdata;
 
@@ -3166,7 +3134,7 @@ static BOOL list_servers()
       if (res != 0)
 	{
 	  printf("\nThis machine does not have a browse list\n");
-	  return;
+	  return(False);
 	}
 
       p = rdata;
@@ -3182,8 +3150,6 @@ static BOOL list_servers()
 	{
 	  char *sname = p;
 	  int comment_offset = IVAL(p,22) & 0xFFFF;
-	  fstring typestr="";
-
 	  printf("\t%-16.16s     %s\n",
 		 sname,
 		 comment_offset?rdata+comment_offset-converter:"");
@@ -3308,7 +3274,7 @@ static BOOL open_sockets(int port )
 	  return False;
 	}
 
-      memcpy((char *)&dest_ip,(char *)hp->h_addr,4);
+      putip((char *)&dest_ip,(char *)hp->h_addr);
     }
 
   Client = open_socket_out(&dest_ip, port);
@@ -3402,225 +3368,6 @@ void wait_keyboard(char *buffer)
     }  
 }
 
-#if 0
-/****************************************************************************
-wait for keyboard activity, swallowing network packets
-****************************************************************************/
-void wait_keyboard(char *buffer)
-{
-  fd_set fds;
-  int selrtn;
-  struct timeval timeout;
-
-  while (1) 
-    {
-      extern int Client;
-      FD_ZERO(&fds);
-      FD_SET(Client,&fds);
-      FD_SET(fileno(stdin),&fds);
-
-      do 
-	{
-	  timeout.tv_sec = 20;
-	  timeout.tv_usec = 0;
-	  selrtn = select(255,SELECT_CAST &fds,NULL,NULL,&timeout);
-	}
-      while(selrtn < 0 && errno == EINTR);
-
-      if (FD_ISSET(fileno(stdin),&fds))
-	return;
-
-      if (FD_ISSET(Client,&fds))
-	receive_smb(buffer,0,False);
-
-      chkpath("\\",False);
-    }
-
-}
-#endif
-
-#if 0
-/****************************************************************************
-register a netbios name on the net.
-****************************************************************************/
-BOOL register_name(name_struct *name,struct in_addr *destip,void (*fn)())
-{
-  int count;
-  char *p;
-  char *inbuf = (char *)malloc(BUFFER_SIZE + SAFETY_MARGIN);
-  char *outbuf = (char *)malloc(BUFFER_SIZE + SAFETY_MARGIN);
-  BOOL saved_swap = NeedSwap;
-  static uint16 name_trn_id = 0;
-
-  if (name_trn_id == 0) name_trn_id = getpid() + (time(NULL) % 1000);
-
-  NeedSwap = !big_endian();
-
-  if ((inbuf == NULL) || (outbuf == NULL)) 
-    {
-      NeedSwap = saved_swap;
-      return False;
-    }
-
-  name_trn_id++;
-
-  DEBUG(1,("Registering name %s (%s) nb_flags=0x%x\n",
-	name->name, inet_ntoa(name->ip) ,name->nb_flags));
-
-  SSVAL(outbuf,0,name_trn_id);
-  CVAL(outbuf,2) = (0x5<<3) | 0x1;
-  CVAL(outbuf,3) = (1<<4) | 0x0;
-  SSVAL(outbuf,4,1);
-  SSVAL(outbuf,6,0);
-  SSVAL(outbuf,8,0);
-  SSVAL(outbuf,10,1);  
-  p = outbuf+12;
-  name_mangle(name->name,p);
-  p += name_len(p);
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  p += 4;
-  CVAL(p,0) = 0xC0;
-  CVAL(p,1) = 12;
-  p += 2;
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  SIVAL(p,4,0); /* my own ttl */
-  SSVAL(p,8,6);
-  CVAL(p,10) = name->nb_flags;
-  CVAL(p,11) = 0;
-  p += 12;
-  memcpy(p,(char *)&(name->ip),4);
-  p += 4;
-
-  count = 3;
-  while (count--)
-    {
-      DEBUG(3,("Sending reg request for %s at (%s)\n",
-	    name->name,inet_ntoa(name->ip)));
-
-
-      show_nmb(outbuf);
-      if (!send_nmb(outbuf,nmb_len(outbuf),destip))
-	{
-	  free(inbuf);free(outbuf);
-	  NeedSwap = saved_swap;
-	  return False;
-	}
-
-      if (receive_nmb(inbuf,10))
-	{
-          int rec_name_trn_id = SVAL(inbuf,0);
-	  int opcode = CVAL(inbuf,2) >> 3;
-	  int nm_flags = ((CVAL(inbuf,2) & 0x7) << 4) + (CVAL(inbuf,3)>>4);
-	  int rcode = CVAL(inbuf,3) & 0xF;
-
-	  /* is it a negative response to our request? */
-	  if ((rec_name_trn_id == name_trn_id) && 
-              (opcode == 21 && nm_flags == 0x58 && rcode != 0 && rcode != 7))
-	    {
-	      char qname[100];
-	      name_extract(inbuf,12,qname,False);
-	      if (name_equal(qname,name->name))
-		{
-		  DEBUG(0,("Someone (%s) gave us a negative name regregistration response!\n",
-			inet_ntoa(lastip)));
-		  free(inbuf);free(outbuf);
-		  NeedSwap = saved_swap;
-		  return False;
-		}
-	      else
-		{
-		  DEBUG(0,("%s gave negative name regregistration for %s??\n",
-			inet_ntoa(lastip),qname));
-		}		  
-	    }	  
-	  
-	  /* it's something else - process it anyway, unless we are running
-	   as a daemon. This is necessary as we may have been started by a 
-	   name query of our name arriving on port 137 (often happens) */
-	  if (fn)
-	    {
-	      show_nmb(inbuf);
-	      fn(inbuf,outbuf + nmb_len(outbuf));
-	    }
-	}
-    }
-
-  /* increment the packet id */
-  name_trn_id++;
-
-  /* don't demand on groups */
-  if ((name->nb_flags & 0x80) != 0)
-    {
-      free(inbuf);free(outbuf);
-      NeedSwap = saved_swap;
-      return(True);
-    }
-
-  /* no negative replies, send a demand */
-  p = outbuf;
-  SSVAL(outbuf,0,name_trn_id);
-  CVAL(outbuf,2) = (0x5<<3);
-  CVAL(outbuf,3) = (1<<4) | 0x0;
-  SSVAL(outbuf,4,1);
-  SSVAL(outbuf,6,0);
-  SSVAL(outbuf,8,0);
-  SSVAL(outbuf,10,1);  
-  p = outbuf+12;
-  name_mangle(name->name,p);
-  p += name_len(p);
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  p += 4;
-  CVAL(p,0) = 0xC0;
-  CVAL(p,1) = 12;
-  p += 2;
-  SSVAL(p,0,0x20);
-  SSVAL(p,2,0x1);
-  SIVAL(p,4,0); /* my own ttl */
-  SSVAL(p,8,6);
-  CVAL(p,10) = name->nb_flags;
-  CVAL(p,11) = 0;
-  p += 12;
-  memcpy(p,(char *)&(name->ip),4);
-  p += 4;
-  
-  DEBUG(3,("Sending reg demand for %s at (%s)\n",
-	name->name,inet_ntoa(name->ip)));
-
-  show_nmb(outbuf);
-
-  {
-    BOOL ret = send_nmb(outbuf,nmb_len(outbuf),destip);
-
-    free(inbuf);free(outbuf);
-  
-    NeedSwap = saved_swap;
-    return(ret);
-  }
-}
-
-
-/****************************************************************************
-try and register my own netbios name with a unicast
-****************************************************************************/
-void register_myname(void)
-{
-  name_struct name;
-
-  name.valid = True;
-  strcpy(name.name,myname);
-  strupper(name.name);
-  strcpy(name.flags,"");
-  name.ip = myip;
-  name.ttl = 0;
-  name.nb_flags = 0;
-
-  register_name(&name,&dest_ip,NULL);
-}
-#endif
-
 
 /****************************************************************************
 close and open the connection again
@@ -3664,10 +3411,6 @@ BOOL process(void )
   
   memset(OutBuffer,0,smb_size);
 
-#if 0
-  register_myname();
-#endif
-
   if (!send_login(InBuffer,OutBuffer,True,True))
     return(False);
 
@@ -3882,7 +3625,7 @@ int main(int argc,char *argv[])
       case 'I':
 	{
 	  unsigned long a = interpret_addr(optarg);
-	  memcpy((char *)&dest_ip,(char *)&a,sizeof(a));
+	  putip((char *)&dest_ip,(char *)&a);
 	  have_ip = True;
 	}
 	break;
@@ -3932,8 +3675,6 @@ int main(int argc,char *argv[])
     }
 
 
-  NeedSwap = big_endian();
-  
   DEBUG(3,("%s client started (version %s)\n",timestring(),VERSION));
 
   if (DEBUGLEVEL > 100)
@@ -3950,11 +3691,6 @@ int main(int argc,char *argv[])
 
   get_machine_info();
 
-#if 0
-  /* Read the broadcast address from the interface */
-  get_broadcast(&myip,&bcast_ip,&Netmask);
-#endif
-
   get_myname(*myname?NULL:myname,&myip);  
   strupper(myname);
   
