@@ -30,23 +30,29 @@ extern connection_struct Connections[];
 
 
 
+uint32 dircounter = 0;
+
+
+#define NUMDIRPTRS 256
+
+
 static struct dptr_struct
 {
-  int key;
   int pid;
   int cnum;
+  uint32 lastused;
   void *ptr;
   BOOL valid;
   BOOL finished;
   BOOL expect_close;
   char *wcard; /* Field only used for lanman2 trans2_findfirst/next searches */
   uint16 attr; /* Field only used for lanman2 trans2_findfirst/next searches */
-  pstring path;
+  char *path;
 }
-dirptrs[MAXDIR];
+dirptrs[NUMDIRPTRS];
 
-static int next_key = 1;
 
+static int dptrs_open = 0;
 
 /****************************************************************************
 initialise the dir array
@@ -57,24 +63,64 @@ void init_dptrs(void)
   int i;
 
   if (dptrs_init) return;
-  for (i=0;i<MAXDIR;i++)    
+  for (i=0;i<NUMDIRPTRS;i++)    
     {
       dirptrs[i].valid = False;
       dirptrs[i].wcard = NULL;
       dirptrs[i].ptr = NULL;
+      string_init(&dirptrs[i].path,"");
     }
   dptrs_init = True;
 }
 
 /****************************************************************************
-get the dir ptr for a dir index
+idle a dptr - the directory is closed but the control info is kept
 ****************************************************************************/
-void *dptr_get(int key)
+static void dptr_idle(int key)
+{
+  if (dirptrs[key].valid && dirptrs[key].ptr) {
+    DEBUG(4,("Idling dptr key %d\n",key));
+    dptrs_open--;
+    CloseDir(dirptrs[key].ptr);
+    dirptrs[key].ptr = NULL;
+  }    
+}
+
+/****************************************************************************
+idle the oldest dptr
+****************************************************************************/
+static void dptr_idleoldest(void)
 {
   int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      return(dirptrs[i].ptr);
+  uint32 old=dircounter+1;
+  int oldi=-1;
+  for (i=0;i<NUMDIRPTRS;i++)
+    if (dirptrs[i].valid && dirptrs[i].ptr && dirptrs[i].lastused < old) {
+      old = dirptrs[i].lastused;
+      oldi = i;
+    }
+  if (oldi != -1)
+    dptr_idle(oldi);
+  else
+    DEBUG(0,("No dptrs available to idle??\n"));
+}
+
+/****************************************************************************
+get the dir ptr for a dir index
+****************************************************************************/
+void *dptr_get(int key,uint32 lastused)
+{
+  if (dirptrs[key].valid) {
+    if (lastused) dirptrs[key].lastused = lastused;
+    if (!dirptrs[key].ptr) {
+      if (dptrs_open >= MAXDIR)
+	dptr_idleoldest();
+      DEBUG(4,("Reopening dptr key %d\n",key));
+      if ((dirptrs[key].ptr = OpenDir(dirptrs[key].path)))
+	dptrs_open++;
+    }
+    return(dirptrs[key].ptr);
+  }
   return(NULL);
 }
 
@@ -83,10 +129,8 @@ get the dir path for a dir index
 ****************************************************************************/
 char *dptr_path(int key)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      return(dirptrs[i].path);
+  if (dirptrs[key].valid)
+    return(dirptrs[key].path);
   return(NULL);
 }
 
@@ -95,10 +139,8 @@ get the dir wcard for a dir index (lanman2 specific)
 ****************************************************************************/
 char *dptr_wcard(int key)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      return(dirptrs[i].wcard);
+  if (dirptrs[key].valid)
+    return(dirptrs[key].wcard);
   return(NULL);
 }
 
@@ -106,32 +148,26 @@ char *dptr_wcard(int key)
 set the dir wcard for a dir index (lanman2 specific)
 Returns 0 on ok, 1 on fail.
 ****************************************************************************/
-int dptr_set_wcard(int key, char *wcard)
+BOOL dptr_set_wcard(int key, char *wcard)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	dirptrs[i].wcard = wcard;
-	return 0;
-      }
-  return 1;
+  if (dirptrs[key].valid) {
+    dirptrs[key].wcard = wcard;
+    return True;
+  }
+  return False;
 }
 
 /****************************************************************************
 set the dir attrib for a dir index (lanman2 specific)
 Returns 0 on ok, 1 on fail.
 ****************************************************************************/
-int dptr_set_attr(int key, uint16 attr)
+BOOL dptr_set_attr(int key, uint16 attr)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	dirptrs[i].attr = attr;
-	return 0;
-      }
-  return 1;
+  if (dirptrs[key].valid) {
+    dirptrs[key].attr = attr;
+    return True;
+  }
+  return False;
 }
 
 /****************************************************************************
@@ -139,122 +175,28 @@ get the dir attrib for a dir index (lanman2 specific)
 ****************************************************************************/
 uint16 dptr_attr(int key)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      return(dirptrs[i].attr);
+  if (dirptrs[key].valid)
+    return(dirptrs[key].attr);
   return(0);
 }
 
-
-
-/****************************************************************************
-check a key
-****************************************************************************/
-BOOL key_ok(int key)
-{
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      return(True);
-  return(False);
-}
-
-/****************************************************************************
-prompte a dptr (to make it recently used)
-****************************************************************************/
-void dptr_promote(int key)
-{
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	struct dptr_struct d;
-	int j;
-	d = dirptrs[i];
-
-	for (j=i;j>0;j--)
-	  dirptrs[j] = dirptrs[j-1];
-	dirptrs[0] = d;
-	return;
-      }
-}
-
-
-/****************************************************************************
-demote a dptr (to make it likely to get closed)
-****************************************************************************/
-void dptr_demote(int key)
-{
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	struct dptr_struct d;
-	int j;
-	d = dirptrs[i];
-	for (j=i;j<(MAXDIR-1) && dirptrs[j].valid;j++)
-	  dirptrs[j] = dirptrs[j+1];
-	dirptrs[j] = d;
-	return;
-      }
-}
-
-
-/****************************************************************************
-find a free key
-****************************************************************************/
-int find_key(int start)
-{
-  int key;
-
-  for (key=start;key<256;key++)
-    if (!key_ok(key))
-      return(key);
-
-  for (key=1;key<start;key++)
-    if (!key_ok(key))
-      return(key);
-
-  DEBUG(0,("ERROR: Out of dirptr keys!\n"));
-  return 1;
-}       
-
-/****************************************************************************
-set the dptr finished flag
-****************************************************************************/
-void dptr_finished(unsigned int key,BOOL finished)
-{
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	dirptrs[i].finished = finished;
-	break;
-      }
-}
-  
-
 /****************************************************************************
 close a dptr
 ****************************************************************************/
 void dptr_close(int key)
 {
-  int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && dirptrs[i].key == key)
-      {
-	DEBUG(6,("closing dptr_num %d\n",key));
-	if (dirptrs[i].ptr)
-	  CloseDir(dirptrs[i].ptr);
-	/* Lanman 2 specific code */
-	if (dirptrs[i].wcard)
-	  free(dirptrs[i].wcard);
-	dirptrs[i].valid = False;
-	next_key = key+1;
-	if (next_key > 255) next_key = 1;
-	return;
-      }
+  if (dirptrs[key].valid) {
+    DEBUG(4,("closing dptr key %d\n",key));
+    if (dirptrs[key].ptr) {
+      CloseDir(dirptrs[key].ptr);
+      dptrs_open--;
+    }
+    /* Lanman 2 specific code */
+    if (dirptrs[key].wcard)
+      free(dirptrs[key].wcard);
+    dirptrs[key].valid = False;
+    string_set(&dirptrs[key].path,"");
+  }
 }
 
 /****************************************************************************
@@ -263,17 +205,20 @@ close all dptrs for a cnum
 void dptr_closecnum(int cnum)
 {
   int i;
-  for (i=0;i<MAXDIR;i++)
+  for (i=0;i<NUMDIRPTRS;i++)
     if (dirptrs[i].valid && dirptrs[i].cnum == cnum)
-      {
-	DEBUG(6,("closing dptr_num %d\n",dirptrs[i].key));
-	if (dirptrs[i].ptr)
-	  CloseDir(dirptrs[i].ptr);
-	/* Lanman 2 specific code */
-	if (dirptrs[i].wcard)
-	  free(dirptrs[i].wcard);
-	dirptrs[i].valid = False;
-      }
+      dptr_close(i);
+}
+
+/****************************************************************************
+idle all dptrs for a cnum
+****************************************************************************/
+void dptr_idlecnum(int cnum)
+{
+  int i;
+  for (i=0;i<NUMDIRPTRS;i++)
+    if (dirptrs[i].valid && dirptrs[i].cnum == cnum && dirptrs[i].ptr)
+      dptr_idle(i);
 }
 
 /****************************************************************************
@@ -282,158 +227,16 @@ close a dptr that matches a given path, only if it matches the pid also
 void dptr_closepath(char *path,int pid)
 {
   int i;
-  for (i=0;i<MAXDIR;i++)
-    if (dirptrs[i].valid && pid == dirptrs[i].pid)
-      {      
-	DEBUG(5,("closepath [%s] [%s]\n",path,dirptrs[i].path));
-	if (strequal(dirptrs[i].path,path))
-	  {
-	    DEBUG(6,("closing dptr_num %d\n",dirptrs[i].key));
-	    if (dirptrs[i].ptr)
-	      CloseDir(dirptrs[i].ptr);
-	    if (dirptrs[i].wcard)
-	      free(dirptrs[i].wcard);
-	    dirptrs[i].valid = False;
-	    next_key = dirptrs[i].key+1;
-	    if (next_key > 255) next_key = 1;
-	  }
-      }
-}
-
-/****************************************************************************
-create a new dir ptr
-****************************************************************************/
-int dptr_create(int cnum,char *path, BOOL expect_close,int pid)
-{
-  int i,key;
-  void *p = Connections[cnum].dirptr;
-
-  for (i=0;i<MAXDIR;i++)
-    if (!dirptrs[i].valid)
-      break;
-
-  /* as a second option, grab one not marked for expect_close but marked 
-     as finished */
-  if (i == MAXDIR)
-    for (i=MAXDIR-1;i>=0;i--)
-      if (dirptrs[i].finished && !dirptrs[i].expect_close)
-	{
-	  dptr_close(dirptrs[i].key);	
-	  break;
-	}
-
-  /* as a third option, grab one not marked for expect_close */
-  if (i < 0)
-    for (i=MAXDIR-1;i>=0;i--)
-      if (!dirptrs[i].expect_close)
-	{
-	  dptr_close(dirptrs[i].key);	
-	  break;
-	}
-
-  if (i < 0)
-    {
-      /* All dirptrs are used by uncloseable SMBffirsts */
-      DEBUG(0,("Error - all dirptrs in use by SMBffirsts.\n"));
-      return(-1);
-    }
-
-  dirptrs[i].ptr = p;
-  strcpy(dirptrs[i].path,path);
-  key = find_key(next_key);
-  dirptrs[i].key = key;
-  dirptrs[i].valid = True;
-  dirptrs[i].finished = False;
-  dirptrs[i].cnum = cnum;
-  dirptrs[i].pid = pid;
-  dirptrs[i].expect_close = expect_close;
-  dirptrs[i].wcard = NULL; /* Only used in lanman2 searches */
-  dirptrs[i].attr = 0; /* Only used in lanman2 searches */
-
-  dptr_promote(key);
-
-  DEBUG(3,("creating new dirptr %d (0x%x) for path %s, expect_close = %d\n",key,p,path,
-	expect_close));  
-
-  return(key);
-}
-
-/****************************************************************************
-fill the 5 byte server reserved dptr field
-****************************************************************************/
-BOOL dptr_fill(char *buf,unsigned int key)
-{
-  void *p = dptr_get(key);
-  int offset;
-  if (!p)
-    {
-      DEBUG(1,("filling null dirptr %d\n",key));
-      return(False);
-    }
-  offset = TellDir(p);
-  DEBUG(6,("fill on dirptr 0x%x now at %d\n",p,offset));
-  buf[0] = key;
-  SIVAL(buf,1,offset);
-  DEBUG(6,("filled dirptr %d at offset %d\n",key,offset));
-  return(True);
-}
-
-
-/****************************************************************************
-return True is the offset is at zero
-****************************************************************************/
-BOOL dptr_zero(char *buf)
-{
-  return(IVAL(buf,0) == 0);
-}
-
-/****************************************************************************
-fetch the dir ptr and seek it given the 5 byte server field
-****************************************************************************/
-void *dptr_fetch(char *buf,int *num)
-{
-  unsigned int key = *(unsigned char *)buf;
-  void *p = dptr_get(key);
-  int offset;
-  if (!p)
-    {
-      DEBUG(3,("fetched null dirptr %d\n",key));
-      return(NULL);
-    }
-  *num = key;
-  offset = IVAL(buf,1);
-  SeekDir(p,offset);
-  dptr_promote(key);
-  DEBUG(3,("fetching dirptr %d for path %s\n",key,dptr_path(key)));
-  return(p);
-}
-
-/****************************************************************************
-fetch the dir ptr and seek it given the lanman2 parameter block
-****************************************************************************/
-void *dptr_fetch_lanman2(char *params,int dptr_num)
-{
-  void *p = dptr_get(dptr_num);
-  uint32 resume_key = SVAL(params,6);
-  BOOL uses_resume_key = BITSETW(params+10,2);
-  BOOL continue_bit = BITSETW(params+10,3);
-
-  if (!p)
-    {
-      DEBUG(3,("fetched null dirptr %d\n",dptr_num));
-      return(NULL);
-    }
-  if(uses_resume_key && !continue_bit)
-    SeekDir(p,resume_key);
-  dptr_promote(dptr_num);
-  DEBUG(3,("fetching dirptr %d for path %s\n",dptr_num,dptr_path(dptr_num)));
-  return(p);
+  for (i=0;i<NUMDIRPTRS;i++)
+    if (dirptrs[i].valid && pid == dirptrs[i].pid &&
+	strequal(dirptrs[i].path,path))
+      dptr_close(i);
 }
 
 /****************************************************************************
   start a directory listing
 ****************************************************************************/
-BOOL start_dir(int cnum,char *directory)
+static BOOL start_dir(int cnum,char *directory)
 {
   DEBUG(5,("start_dir cnum=%d dir=%s\n",cnum,directory));
 
@@ -444,10 +247,152 @@ BOOL start_dir(int cnum,char *directory)
     directory = ".";
 
   Connections[cnum].dirptr = OpenDir(directory);
-
-  string_set(&Connections[cnum].dirpath,directory);
+  if (Connections[cnum].dirptr) {    
+    dptrs_open++;
+    string_set(&Connections[cnum].dirpath,directory);
+    return(True);
+  }
   
-  return(Connections[cnum].dirptr != NULL);
+  return(False);
+}
+
+
+/****************************************************************************
+create a new dir ptr
+****************************************************************************/
+int dptr_create(int cnum,char *path, BOOL expect_close,int pid)
+{
+  int i;
+  uint32 old;
+  int oldi;
+
+  if (!start_dir(cnum,path))
+    return(-1);
+
+  if (dptrs_open >= MAXDIR)
+    dptr_idleoldest();
+
+  for (i=0;i<NUMDIRPTRS;i++)
+    if (!dirptrs[i].valid)
+      break;
+  if (i == NUMDIRPTRS) i = -1;
+
+
+  /* as a 2nd option, grab the oldest not marked for expect_close */
+  if (i == -1) {
+    old=dircounter+1;
+    oldi=-1;
+    for (i=0;i<NUMDIRPTRS;i++)
+      if (!dirptrs[i].expect_close && dirptrs[i].lastused < old) {
+	old = dirptrs[i].lastused;
+	oldi = i;
+      }
+    i = oldi;
+  }
+
+  /* a 3rd option - grab the oldest one */
+  if (i == -1) {
+    old=dircounter+1;
+    oldi=-1;
+    for (i=0;i<NUMDIRPTRS;i++)
+      if (dirptrs[i].lastused < old) {
+	old = dirptrs[i].lastused;
+	oldi = i;
+      }
+    i = oldi;
+  }
+
+  if (i == -1) {
+    DEBUG(0,("Error - all dirptrs in use??\n"));
+    return(-1);
+  }
+
+  if (dirptrs[i].valid)
+    dptr_close(i);
+
+  dirptrs[i].ptr = Connections[cnum].dirptr;
+  string_set(&dirptrs[i].path,path);
+  dirptrs[i].lastused = dircounter++;
+  dirptrs[i].finished = False;
+  dirptrs[i].cnum = cnum;
+  dirptrs[i].pid = pid;
+  dirptrs[i].expect_close = expect_close;
+  dirptrs[i].wcard = NULL; /* Only used in lanman2 searches */
+  dirptrs[i].attr = 0; /* Only used in lanman2 searches */
+  dirptrs[i].valid = True;
+
+  DEBUG(3,("creating new dirptr %d for path %s, expect_close = %d\n",
+	   i,path,expect_close));  
+
+  return(i);
+}
+
+/****************************************************************************
+fill the 5 byte server reserved dptr field
+****************************************************************************/
+BOOL dptr_fill(char *buf1,unsigned int key)
+{
+  unsigned char *buf = (unsigned char *)buf1;
+  void *p = dptr_get(key,0);
+  int offset;
+  if (!p) {
+    DEBUG(1,("filling null dirptr %d\n",key));
+    return(False);
+  }
+  offset = TellDir(p);
+  DEBUG(6,("fill on key %d dirptr 0x%x now at %d\n",key,p,offset));
+  buf[0] = key;
+  SIVAL(buf,1,offset + 1);
+  return(True);
+}
+
+
+/****************************************************************************
+return True is the offset is at zero
+****************************************************************************/
+BOOL dptr_zero(char *buf)
+{
+  return(IVAL(buf,0) <= 1);
+}
+
+/****************************************************************************
+fetch the dir ptr and seek it given the 5 byte server field
+****************************************************************************/
+void *dptr_fetch(char *buf,int *num)
+{
+  unsigned int key = *(unsigned char *)buf;
+  void *p = dptr_get(key,dircounter++);
+  int offset;
+  if (!p) {
+    DEBUG(3,("fetched null dirptr %d\n",key));
+    return(NULL);
+  }
+  *num = key;
+  offset = IVAL(buf,1) - 1;
+  SeekDir(p,offset<0?0:offset);
+  DEBUG(3,("fetching dirptr %d for path %s at offset %d\n",
+	   key,dptr_path(key),offset));
+  return(p);
+}
+
+/****************************************************************************
+fetch the dir ptr and seek it given the lanman2 parameter block
+****************************************************************************/
+void *dptr_fetch_lanman2(char *params,int dptr_num)
+{
+  void *p = dptr_get(dptr_num,dircounter++);
+  uint32 resume_key = SVAL(params,6);
+  BOOL uses_resume_key = BITSETW(params+10,2);
+  BOOL continue_bit = BITSETW(params+10,3);
+
+  if (!p) {
+    DEBUG(3,("fetched null dirptr %d\n",dptr_num));
+    return(NULL);
+  }
+  if(uses_resume_key && !continue_bit)
+    SeekDir(p,resume_key);
+  DEBUG(3,("fetching dirptr %d for path %s\n",dptr_num,dptr_path(dptr_num)));
+  return(p);
 }
 
 /****************************************************************************
@@ -541,6 +486,7 @@ BOOL get_dir_entry(int cnum,char *mask,int dirtype,char *fname,int *size,int *mo
 typedef struct
 {
   int pos;
+  BOOL end;
   void *dirptr;
 } Dir;
 
@@ -553,12 +499,12 @@ void *OpenDir(char *name)
   void *p = opendir(name);
   if (!p) return(NULL);
   dirp = (Dir *)malloc(sizeof(Dir));
-  if (!dirp)
-    {
-      closedir(p);
-      return(NULL);
-    }
+  if (!dirp) {
+    closedir(p);
+    return(NULL);
+  }
   dirp->pos = 0;
+  dirp->end=False;
   dirp->dirptr = p;
   return((void *)dirp);
 }
@@ -589,14 +535,18 @@ char *ReadDirName(void *p)
   Dir *dirp = (Dir *)p;
   struct DIRECT *ptr;
 
-  if (!dirp) return(NULL);
+  if (!dirp || dirp->end) return(NULL);
   
   ptr = (struct DIRECT *)readdir(dirp->dirptr);
-  if (!ptr) return(NULL);
+  if (!ptr) {
+    dirp->end = True;
+    return(NULL);
+  }
 
 #ifdef NEXT2
   if (telldir(dirp->dirptr) < 0)
     {
+      dirp->end = True;
       DEBUG(3,("ReadDir encountered null - returning NULL\n"));
       return(NULL);
     }
@@ -632,17 +582,20 @@ BOOL SeekDir(void *p,int pos)
 
   if (!dirp) return(False);
 
-  if (pos < dirp->pos)
-    {
-      rewinddir(dirp->dirptr);
-      dirp->pos = 0;
-    }
+  dirp->end = False;
 
-  while (dirp->pos < pos)
-    {
-      if (!readdir(dirp->dirptr)) return(False);
-      dirp->pos++;
+  if (pos < dirp->pos) {
+    rewinddir(dirp->dirptr);
+    dirp->pos = 0;
+  }
+
+  while (dirp->pos < pos) {
+    if (!readdir(dirp->dirptr)) {
+      dirp->end = True;
+      return(False);      
     }
+    dirp->pos++;
+  }
 
   return(True);
 }
