@@ -25,23 +25,68 @@
 */
 
 
+/* the first OS dependent section is to setup what includes will be used
+   the main OS dependent section comes later on 
+*/
 
+#ifdef MIPS
+#define POSIX_H
+#define NO_UTIMEH
+#endif
+
+#ifdef sun386
+#define NO_UTIMEH
+#endif
+
+#ifdef NEXT2
+#define NO_UTIMEH
+#endif
+
+#ifdef NEXT3_0
+#define NO_UTIMEH
+#define NO_UNISTDH
+#endif
+
+#ifdef APOLLO
+#define NO_UTIMEH
+#define NO_SYSMOUNTH
+#define NO_UNISTDH
+#endif
+
+#ifdef M88K_R3
+#define SVR3H
+#endif
+
+#ifdef DNIX
+#define NO_SYSMOUNTH
+#define NO_NETIFH
+#endif
+
+
+#ifdef ISC
+#define SYSSTREAMH
+#endif
+
+
+
+/* here is the general includes section - with some ifdefs generated 
+   by the previous section 
+*/
 #include "local.h"
 #include <stdio.h>
-#ifdef MIPS
+#ifdef POSIX_STDLIBH
 #include <posix/stdlib.h>
 #else
 #include <stdlib.h>
-#endif /* MIPS */
+#endif
 #include <ctype.h>
 #include <time.h>
-#if (!(defined(sun386) || defined(NEXT2) || defined(NEXT3_0) || defined(APOLLO) || defined(MIPS)))
+#ifndef NO_UTIMEH
 #include <utime.h>
 #endif
 #include <sys/types.h>
 
-#ifdef M88K_R3
-#define STATFS4
+#ifdef SVR3H
 #include <sys/statfs.h>
 #include <sys/stream.h>
 #include <netinet/types.h>
@@ -52,14 +97,14 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <stddef.h>
-#ifdef MIPS
+#ifdef POSIX_H
 #include <posix/utime.h>
 #include <bsd/sys/time.h>
 #include <bsd/netinet/in.h>
 #else
 #include <sys/time.h>
 #include <netinet/in.h>
-#endif /* MIPS */
+#endif 
 #include <netdb.h>
 #include <signal.h>
 #include <errno.h>
@@ -67,7 +112,10 @@
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <grp.h>
-#if (!defined(APOLLO) && !defined(DNIX))
+#ifndef NO_RESOURCEH
+#include <sys/resource.h>
+#endif
+#ifndef NO_SYSMOUNTH
 #include <sys/mount.h>
 #endif
 #include <pwd.h>
@@ -76,20 +124,17 @@
 #else
 #include <varargs.h>
 #endif
-#if !defined( NEXT3_0 ) && !defined( APOLLO )
+#ifndef NO_UNISTDH
 #include <unistd.h>
 #endif
-#ifdef CLIX	/* need rusage structure */
-#include <sys/resource.h>
-#endif	/* end CLIX */
 #include <sys/wait.h>
-#ifdef ISC
+#ifdef SYSSTREAMH
 #include <sys/stream.h>
 #endif
-#ifdef MIPS
+#ifndef NO_NETIFH
+#ifdef POSIX_H
 #include <bsd/net/if.h>
 #else
-#ifndef DNIX
 #include <net/if.h>
 #endif
 #endif
@@ -192,6 +237,7 @@ extern char *sys_errlist[];
 #include <netinet/tcp.h>
 #include <netinet/ip.h>
 #include <string.h>
+#define USE_SETVBUF
 #define HAVE_TIMEZONE
 #define SIGNAL_CAST (void (*)(int))
 #ifndef SYSV
@@ -269,7 +315,7 @@ extern char *sys_errlist[];
 #define USE_WAITPID
 #define HAVE_TIMEZONE
 #define SIGNAL_CAST (void (*)(int))
-#define USE_STATFS4
+#define STATFS4
 #define USE_GETCWD
 #endif
 
@@ -606,6 +652,7 @@ char *strdup (char *);
 #include <dirent.h>
 #include <fcntl.h>
 #include <termios.h>
+#define STATFS4
 #define SYSV
 #define USE_WAITPID
 #define HAVE_TIMEZONE
