@@ -722,7 +722,11 @@ BOOL lp_add_home(char *pszHomename, int iDefaultService, char *pszHomedir)
   if (!(*(iSERVICE(i).szPath)))
     string_set(&iSERVICE(i).szPath,pszHomedir);
   if (!(*(iSERVICE(i).comment)))
-    string_set(&iSERVICE(i).comment,"Home directory of %S");
+    {
+      pstring comment;
+      sprintf(comment,"Home directory of %s",pszHomename);
+      string_set(&iSERVICE(i).comment,comment);
+    }
   iSERVICE(i).bAvailable = True;
   iSERVICE(i).bBrowseable = sDefault.bBrowseable;
 
@@ -1322,7 +1326,8 @@ Returns True on success, False on failure.
 static BOOL do_section(char *pszSectionName)
 {
    BOOL bRetval;
-   BOOL isglobal = (strwicmp(pszSectionName, GLOBAL_NAME) == 0);
+   BOOL isglobal = ((strwicmp(pszSectionName, GLOBAL_NAME) == 0) || 
+		    (strwicmp(pszSectionName, GLOBAL_NAME2) == 0));
    bRetval = False;
 
    /* if we were in a global section then do the local inits */
