@@ -75,7 +75,7 @@ BOOL is_reserved_msdos(char *fname)
 }
 
 
-static BOOL valid_dos_char[256];
+BOOL valid_dos_char[256];
 
 #define VALID_DOS_PCHAR(p) (valid_dos_char[*(unsigned char *)(p)])
 
@@ -299,7 +299,7 @@ BOOL check_mangled_stack(char *s, char *MangledMap)
     {
       strcpy(tmpname,mangled_stack[i]);
       mangle_name_83(tmpname, MangledMap);
-      if (strcmp(tmpname,s) == 0)
+      if (strequal(tmpname,s))
 	{
 	  strcpy(s,mangled_stack[i]);
 	  break;
@@ -309,7 +309,7 @@ BOOL check_mangled_stack(char *s, char *MangledMap)
 	  strcpy(tmpname,mangled_stack[i]);
 	  strcat(tmpname,extension);
 	  mangle_name_83(tmpname, MangledMap);
-	  if (strcmp(tmpname,s) == 0)
+	  if (strequal(tmpname,s))
 	    {
 	      strcpy(s,mangled_stack[i]);
 	      strcat(s,extension);
@@ -335,7 +335,7 @@ string
 void mangle_search_83(char *s,char *home,char *dir, char *MangledMap)
 {
   void *dirptr;
-  struct DIRECT *dptr;
+  char *dname;
   pstring tmpname;
 
   DEBUG(3,("mangle search - searching for %s in %s\n",s,dir));
@@ -350,31 +350,30 @@ void mangle_search_83(char *s,char *home,char *dir, char *MangledMap)
   if (tmpname[strlen(tmpname)-1] == '/')
     strcat(tmpname,".");
 
-  dirptr = (void *)opendir(tmpname);
+  dirptr = OpenDir(tmpname);
 
   if (!dirptr)
     return;
 
-  dptr = ReadDir(dirptr);
-  while (dptr)
+  
+  for (dname = ReadDirName(dirptr); dname; dname = ReadDirName(dirptr))
     {
-      strcpy(tmpname,dptr->d_name);
+      strcpy(tmpname,dname);
       mangle_name_83(tmpname, MangledMap);
-      DEBUG(5,("Mangle: Trying %s gave %s\n",dptr->d_name,tmpname));
+      DEBUG(5,("Mangle: Trying %s gave %s\n",dname,tmpname));
       if (strcmp(tmpname,s) == 0)
 	{
-	  strcpy(s,dptr->d_name);
+	  strcpy(s,dname);
 	  push_mangled_name(s);
-	  closedir(dirptr);
+	  CloseDir(dirptr);
 	  return;
 	}
-      dptr = ReadDir(dirptr);
     }
   
   /* didn't find it */
   strnorm(s);
 
-  closedir(dirptr);
+  CloseDir(dirptr);
 }
 
 
