@@ -407,8 +407,11 @@ BOOL receive_nmb(char *buffer,int timeout)
   if (ret < 0)
     {
       DEBUG(0,("No bytes from client\n"));
-      close_sockets();
-      exit(0);
+      if (!is_daemon)
+	{
+	  close_sockets();
+	  exit(0);
+	}
     }
   
   if (ret <= 1)
