@@ -463,11 +463,20 @@ check if a username is valid
 ****************************************************************************/
 BOOL user_ok(char *user,int snum)
 {
-  char *valid = lp_valid_users(snum);
-  char *invalid = lp_invalid_users(snum);
-  BOOL ret = !user_in_list(user,invalid);
+  pstring valid, invalid;
+  BOOL ret;
+
+  StrnCpy(valid, lp_valid_users(snum), sizeof(pstring));
+  StrnCpy(invalid, lp_invalid_users(snum), sizeof(pstring));
+
+  string_sub(valid,"%S",lp_servicename(snum));
+  string_sub(invalid,"%S",lp_servicename(snum));
+
+  ret = !user_in_list(user,invalid);
+
   if (ret && valid && *valid)
     ret = user_in_list(user,valid);
+
   return(ret);
 }
 
@@ -613,11 +622,13 @@ BOOL authorise_login(int snum,char *user,char *password,BOOL *guest,int vuid)
 	}
 
       /* check the user= fields and the given password */
-      if (!ok && *password && USER(snum))
+      if (!ok && *password && lp_username(snum))
 	{
 	  char *auser;
-	  char *user_list = strdup(USER(snum));
-	  if (!user_list) return(False);
+	  pstring user_list;
+	  StrnCpy(user_list,lp_username(snum),sizeof(pstring));
+
+	  string_sub(user_list,"%S",lp_servicename(snum));
 	  
 	  for (auser=strtok(user_list,LIST_SEP);
 	       auser && !ok;
@@ -643,7 +654,6 @@ BOOL authorise_login(int snum,char *user,char *password,BOOL *guest,int vuid)
 		    }
 		}
 	    }
-	  free(user_list);
 	}
       
     } /* not guest only */
@@ -845,7 +855,6 @@ BOOL server_cryptkey(uint16 *sec_flags,char *crypt_buf,int *crypt_len,uint32 *se
 {
   extern fstring remote_machine;
   extern int Client;
-  extern BOOL NeedSwap;
   int SavedClient = Client;
   char *p;
   int len;
@@ -869,7 +878,7 @@ BOOL server_cryptkey(uint16 *sec_flags,char *crypt_buf,int *crypt_len,uint32 *se
 
   {
     unsigned long a = interpret_addr(desthost);
-    memcpy((char *)&dest_ip,(char *)&a,sizeof(a));
+    putip((char *)&dest_ip,(char *)&a);
   }
 
   password_client = Client = open_socket_out(&dest_ip, port);
@@ -882,7 +891,6 @@ BOOL server_cryptkey(uint16 *sec_flags,char *crypt_buf,int *crypt_len,uint32 *se
   DEBUG(3,("connected to password server\n"));
 
   /* send a session request (RFC 8002) */
-  CVAL(outbuf,0) = 0x81;
 
   /* put in the destination name */
   len = 4;
@@ -895,12 +903,8 @@ BOOL server_cryptkey(uint16 *sec_flags,char *crypt_buf,int *crypt_len,uint32 *se
   name_mangle(remote_machine,p);
   len += name_len(p);
 
-  NeedSwap = !NeedSwap;
-  SSVAL(outbuf,2,len);
-  NeedSwap = !NeedSwap;
-
-  if (len >= (1 << 16))
-    CVAL(outbuf,1) |= 1;
+  _smb_setlen(outbuf,len);
+  CVAL(outbuf,0) = 0x81;
 
   send_smb(outbuf);
   receive_smb(inbuf,0,False);
