@@ -51,7 +51,7 @@ static char *build_print_command(int cnum, char *command, char *syscmd, char *fi
    strcpy(syscmd, tstr);
 
    /* look for "%s" in the string. If there is no %s, we cannot print. */   
-   if (!strstr(syscmd, "%s"))
+   if (!strstr(syscmd, "%s") && !strstr(syscmd, "%f"))
    {
       DEBUG(0,("No placeholder for the filename in the print command "
               "for service %s!\n", SERVICE(snum)));
