@@ -624,10 +624,10 @@ int reply_search(char *inbuf,char *outbuf)
       memcpy(status,smb_buf(inbuf) + 1 + strlen(path) + 4,21);
       memcpy(mask,status+1,11);
       mask[11] = 0;
+      dirtype = CVAL(status,0) & 0x1F;
       Connections[cnum].dirptr = dptr_fetch(status+12,&dptr_num);      
       if (!Connections[cnum].dirptr)
 	goto SearchEmpty;
-      dirtype = CVAL(status,0) & 0x1F;
       string_set(&Connections[cnum].dirpath,dptr_path(dptr_num));
       if (!case_sensitive)
 	strnorm(mask);
@@ -681,7 +681,6 @@ int reply_search(char *inbuf,char *outbuf)
 
 
   DEBUG(5,("mask=%s directory=%s\n",mask,directory));
-  CVAL(smb_buf(outbuf),0) = 5;
   
   if (can_open)
     {
@@ -691,18 +690,9 @@ int reply_search(char *inbuf,char *outbuf)
       
       if (status_len == 0)
 	{
-	  if (!start_dir(cnum,directory))
-	    ok = False;
-	  else
-	    {
-	      dptr_num = dptr_create(cnum,directory,expect_close,SVAL(inbuf,smb_pid));
-	      if (dptr_num < 0)
-		{
-		  if(Connections[cnum].dirptr != NULL)
-		    CloseDir(Connections[cnum].dirptr);
-		  return(ERROR(ERRDOS,ERRnofids));
-		}
-	    }
+	  dptr_num = dptr_create(cnum,directory,expect_close,SVAL(inbuf,smb_pid));
+	  if (dptr_num < 0)
+	    return(ERROR(ERRDOS,ERRnofids));
 	}
 
       DEBUG(4,("dptr_num is %d\n",dptr_num));
@@ -740,7 +730,6 @@ int reply_search(char *inbuf,char *outbuf)
 		    }
 		  p += DIR_STRUCT_SIZE;
 		}
-	      dptr_finished(dptr_num,finished);
 	    }
 	}
     }
@@ -752,8 +741,6 @@ int reply_search(char *inbuf,char *outbuf)
     {
       CVAL(outbuf,smb_rcls) = ERRDOS;
       SSVAL(outbuf,smb_err,ERRnofiles);
-      if (dptr_num >= 0)
-	dptr_demote(dptr_num);
     }
 
   /* If we were called as SMBffirst with smb_search_id == NULL
@@ -765,16 +752,12 @@ int reply_search(char *inbuf,char *outbuf)
       CVAL(outbuf,smb_rcls) = ERRDOS;
       SSVAL(outbuf,smb_err,ERRnofiles);
       /* Also close the dptr - we know it's gone */
-      dptr_demote(dptr_num);
       dptr_close(dptr_num);
     }
 
   /* If we were called as SMBfunique, then we can close the dirptr now ! */
   if(dptr_num >= 0 && CVAL(inbuf,smb_com) == SMBfunique)
-    {
-      dptr_demote(dptr_num);
-      dptr_close(dptr_num);
-    }
+    dptr_close(dptr_num);
 
   SSVAL(outbuf,smb_vwv0,numentries);
   SSVAL(outbuf,smb_vwv1,3 + numentries * DIR_STRUCT_SIZE);
@@ -787,10 +770,10 @@ int reply_search(char *inbuf,char *outbuf)
   if ((! *directory) && dptr_path(dptr_num))
     sprintf(directory,"(%s)",dptr_path(dptr_num));
 
-  DEBUG(4,("%s %s mask=%s directory=%s cnum=%d dirtype=%d numentries=%d\n",
+  DEBUG(4,("%s %s mask=%s path=%s cnum=%d dtype=%d nument=%d of %d\n",
 	timestring(),
 	smb_fn_name(CVAL(inbuf,smb_com)), 
-	mask,directory,cnum,dirtype,numentries));
+	mask,directory,cnum,dirtype,numentries,maxentries));
 
   return(outsize);
 }
@@ -820,12 +803,10 @@ int reply_fclose(char *inbuf,char *outbuf)
 
   memcpy(status,smb_buf(inbuf) + 1 + strlen(path) + 4,21);
 
-  if(dptr_fetch(status+12,&dptr_num))
-    {
-      /*  Close the dptr - we know it's gone */
-      dptr_demote(dptr_num);
-      dptr_close(dptr_num);
-    }
+  if(dptr_fetch(status+12,&dptr_num)) {
+    /*  Close the dptr - we know it's gone */
+    dptr_close(dptr_num);
+  }
 
   SSVAL(outbuf,smb_vwv0,0);
 
