@@ -683,7 +683,7 @@ static int old_umask = 022;
 /****************************************************************************
 load parameters specific to a connection/service
 ****************************************************************************/
-void become_service(int cnum,BOOL do_chdir)
+BOOL become_service(int cnum,BOOL do_chdir)
 {
   extern char magic_char;
   static int last_cnum = -1;
@@ -692,7 +692,7 @@ void become_service(int cnum,BOOL do_chdir)
   if (!OPEN_CNUM(cnum))
     {
       last_cnum = -1;
-      return;
+      return(False);
     }
 
   Connections[cnum].lastused = time(NULL);
@@ -707,11 +707,11 @@ void become_service(int cnum,BOOL do_chdir)
     {
       DEBUG(0,("%s chdir (%s) failed cnum=%d\n",timestring(),
 	    Connections[cnum].connectpath,cnum));     
-      return;
+      return(False);
     }
 
   if (cnum == last_cnum)
-    return;
+    return(True);
 
   last_cnum = cnum;
 
@@ -720,6 +720,7 @@ void become_service(int cnum,BOOL do_chdir)
   case_mangle = lp_casemangle(snum);
   case_sensitive = lp_casesensitive(snum);
   magic_char = lp_magicchar(snum);
+  return(True);
 }
 
 
@@ -1725,20 +1726,26 @@ int make_connection(char *service,char *user,char *password,char *dev,int vuid)
       return(-1);
     }
 
-  num_connections_open++;
-  add_session_user(user);
+  if (ChDir(Connections[cnum].connectpath) != 0)
+    {
+      DEBUG(0,("Can't change directory to %s\n",Connections[cnum].connectpath));
+      Connections[cnum].open = False;
+      return(-1);      
+    }
 
   string_set(&Connections[cnum].origpath,Connections[cnum].connectpath);
 
   /* resolve any soft links early */
   {
     pstring s;
-    ChDir(Connections[cnum].connectpath);
     strcpy(s,Connections[cnum].connectpath);
     GetWd(s);
     string_set(&Connections[cnum].connectpath,s);
     ChDir(Connections[cnum].connectpath);
   }
+
+  num_connections_open++;
+  add_session_user(user);
   
   /* execute any "preexec = " line */
   if (*lp_preexec(SNUM(cnum)))
@@ -2914,7 +2921,8 @@ int switch_message(int type,char *inbuf,char *outbuf,int size,int bufsize,BOOL p
 	    return(ERROR(ERRSRV,ERRaccess));	    
 
 	  /* load service specific parameters */
-	  become_service(cnum,(flags & AS_USER));
+	  if (OPEN_CNUM(cnum) && !become_service(cnum,(flags & AS_USER)))
+	    return(ERROR(ERRSRV,ERRaccess));
 
 	  /* does this protocol need to be run as guest? */
 	  if ((flags & AS_GUEST) && (!become_guest() || !check_access(-1)))
@@ -3168,7 +3176,7 @@ void process(void )
 		/* close dirptrs on connections that are idle for 
 		   5 minutes */
 		if ((t-Connections[i].lastused)>5*60)
-		  dptr_closecnum(i);
+		  dptr_idlecnum(i);
 
 		/* close connections that are idle for deadtime 
 		 and have no files open */
@@ -3376,6 +3384,18 @@ int main(int argc,char *argv[])
 
   GetWd(OriginalDir);
   get_machine_info();
+
+#ifdef RLIMIT_NOFILE
+  {
+    struct rlimit rlp;
+    getrlimit(RLIMIT_NOFILE, &rlp);
+    rlp.rlim_cur = (MAX_OPEN_FILES>rlp.rlim_max)? rlp.rlim_max:MAX_OPEN_FILES;
+    setrlimit(RLIMIT_NOFILE, &rlp);
+    getrlimit(RLIMIT_NOFILE, &rlp);
+    DEBUG(3,("Maximum number of open files per session is %d\n",rlp.rlim_cur));
+  }
+#endif
+
   
   DEBUG(2,("uid=%d gid=%d euid=%d egid=%d\n",
 	getuid(),getgid(),geteuid(),getegid()));
