@@ -522,6 +522,7 @@ struct from_host {
 };
 
 /* and a few prototypes */
+void dptr_idlecnum(int cnum);
 void dptr_closecnum(int cnum);
 void init_dptrs(void);
 void fault_setup();
@@ -536,7 +537,7 @@ int TellDir(void *p);
 int write_data(int fd,char *buffer,int N);
 BOOL server_cryptkey(uint16 *sec_flags,char *crypt_buf,int *crypt_len,uint32 *sesskey);
 BOOL server_validate(char *username,uint32 sesskey,int passlen,char *pass);
-void become_service(int cnum,BOOL do_chdir);
+BOOL become_service(int cnum,BOOL do_chdir);
 BOOL snum_used(int snum);
 BOOL reload_services(BOOL test);
 void reopen_logs(void);
@@ -547,7 +548,6 @@ BOOL do_match(char *str, char *regexp, int case_sig);
 BOOL is_a_socket(int fd);
 void _smb_setlen(char *buf,int len);
 void valid_initialise(void);
-void dptr_finished(unsigned int key,BOOL finished);
 BOOL is_8_3(char *fname);
 BOOL is_mangled(char *s);
 void standard_sub(int cnum,char *s);
@@ -581,14 +581,12 @@ int get_printqueue(int snum,int cnum,print_queue_struct **queue,print_status_str
 int find_free_connection(int hash);
 void parse_connect(char *buf,char *service,char *user,char *password,char *dev);
 int make_connection(char *service,char *user,char *password,char *dev,int vuid);
-void *dptr_get(int key);
+void *dptr_get(int key,uint32 lastused);
 char *dptr_path(int key);
 char *dptr_wcard(int key);
-int dptr_set_wcard(int key, char *wcard);
-int dptr_set_attr(int key, uint16 attr);
+BOOL dptr_set_wcard(int key, char *wcard);
+BOOL dptr_set_attr(int key, uint16 attr);
 uint16 dptr_attr(int key);
-void dptr_promote(int key);
-void dptr_demote(int key);
 void dptr_close(int key);
 void dptr_closepath(char *path,int pid);
 int dptr_create(int cnum,char *path, BOOL expect_close,int pid);
@@ -616,7 +614,6 @@ void disk_free(char *path,int *bsize,int *dfree,int *dsize);
 int unix_error_packet(char *inbuf,char *outbuf,int def_class,uint32 def_code,int line);
 time_t make_unix_date2(void *date_ptr);
 int cached_error_packet(char *inbuf,char *outbuf,int fnum,int line);
-BOOL start_dir(int cnum,char *directory);
 mode_t unix_mode(int cnum,int dosmode);
 BOOL check_name(char *name,int cnum);
 int error_packet(char *inbuf,char *outbuf,int error_class,uint32 error_code,int line);
