@@ -23,42 +23,65 @@
  */
 
 #include "includes.h"
-#include "smb.h"
+#include "loadparm.h"
 
 struct connect_record crec;
+extern int DEBUGLEVEL;
+extern FILE *dbf;
 
 int main(int argc, char *argv[])
 {
   FILE *f;
-  pstring fname;
-  int uid;
+  pstring fname="";
+  int uid, c;
+  static pstring servicesf = CONFIGFILE;
+  extern char *optarg;
+  int verbose = 0;
 
-  strcpy(fname,LOCKDIR);
+  DEBUGLEVEL = 0;
+  dbf = fopen("/dev/null","w");
+
+  if (getuid() != geteuid()) {
+    printf("smbstatus should not be run setuid\n");
+    return(1);
+  }
+
+  while ((c = getopt(argc, argv, "ds:")) != EOF) {
+    switch (c) {
+    case 'd':
+      verbose = 1;
+      break;
+    case 's':
+      strcpy(servicesf, optarg);
+      break;
+    default:
+      fprintf(stderr, "Usage: %s [-d] [-s configfile]\n", *argv);
+      return (-1);
+    }
+  }
+
+  if (!lp_load(servicesf)) {
+    fprintf(stderr, "Can't load %s - run testparm to debug it\n", servicesf);
+    return (-1);
+  }
+
+  if (verbose) {
+    printf("using configfile = %s\n", servicesf);
+    printf("lockdir = %s\n", *lp_lockdir() ? lp_lockdir() : "NULL");
+  }
+
+  strcpy(fname,lp_lockdir());
+  standard_sub_basic(fname);
   trim_string(fname,"","/");
   strcat(fname,"/STATUS..LCK");
 
-  /* if we're running suid then don't allow them to specify the lock 
-     file */
-  if (argc > 1)
-    {
-      if (getuid() == geteuid())
-	strcpy(fname,argv[1]);
-      else
-	printf("ignoring filename %s as process is suid\n",argv[1]);
-    }
-  
-
   f = fopen(fname,"r");
-  if (!f)
-    {
-      printf("Couldn't open status file %s\n",fname);
-      printf("Perhaps you don't have status=yes in [globals] ?\n");
-      printf("or maybe you need to give this program the name of your status file\n");
-      return(0);
-    }
-
-  if (getuid() != 0)
-    setuid(0);
+  if (!f) {
+    printf("Couldn't open status file %s\n",fname);
+    if (!lp_status(-1))
+      printf("You need to have status=yes in your smb config file\n");
+    return(0);
+  }
 
   uid = getuid();
 
