@@ -312,6 +312,9 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
   uint32 mdate=0, adate=0, cdate=0;
   int name_offset;
   int space_for_this_entry;
+  BOOL isrootdir = (strequal(Connections[cnum].dirpath,"./") ||
+		    strequal(Connections[cnum].dirpath,".") ||
+		    strequal(Connections[cnum].dirpath,"/"));
 
   *fname = 0;
   *out_of_space = False;
@@ -350,8 +353,11 @@ static int get_lanman2_dir_entry(int cnum,char *path_mask,int dirtype,int info_l
 
       if(mask_match(fname, mask, case_sensitive, True))
 	{
-	  if (dont_descend &&
-	      !strequal(fname,".") && !strequal(fname,".."))
+	  BOOL isdots = (strequal(fname,"..") || strequal(fname,"."));
+	  if (dont_descend && !isdots)
+	    continue;
+	  
+	  if (isrootdir && isdots)
 	    continue;
 
 	  strcpy(pathreal,Connections[cnum].dirpath);
@@ -547,23 +553,15 @@ int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum,
   if(params == NULL)
     return(ERROR(ERRDOS,ERRnomem));
 
-  if(!start_dir(cnum,directory))
-    return(ERROR(ERRDOS,ERRbadpath));
-	
   dptr_num = dptr_create(cnum,directory, True ,SVAL(inbuf,smb_pid));
   if (dptr_num < 0)
-    {
-      if(Connections[cnum].dirptr != NULL)
-	CloseDir(Connections[cnum].dirptr);
-      return(ERROR(ERRDOS,ERRnofids));
-    }
+    return(ERROR(ERRDOS,ERRbadpath));
 
   /* Save the wildcard match and attribs we are using on this directory - needed as
      lanman2 assumes these are being saved between calls */
 
   if(!(wcard = strdup(mask)))
     {
-      dptr_demote(dptr_num);
       dptr_close(dptr_num);
       return(ERROR(ERRDOS,ERRnomem));
     }
@@ -614,7 +612,6 @@ int call_trans2findfirst(char *inbuf, char *outbuf, int bufsize, int cnum,
   /* Check if we can close the dirptr */
   if(close_after_first || (finished && close_if_end))
     {
-      dptr_demote(dptr_num);
       dptr_close(dptr_num);
       DEBUG(5,("call_trans2findfirst - (2) closing dptr_num %d\n", dptr_num));
       dptr_num = -1;
@@ -779,7 +776,6 @@ int call_trans2findnext(char *inbuf, char *outbuf, int length, int bufsize,
   /* Check if we can close the dirptr */
   if(close_after_request || (finished && close_if_end))
     {
-      dptr_demote(dptr_num);
       dptr_close(dptr_num); /* This frees up the saved mask */
       DEBUG(5,("call_trans2findnext: closing dptr_num = %d\n", dptr_num));
       dptr_num = -1;
@@ -1361,21 +1357,11 @@ int reply_findclose(char *inbuf,char *outbuf,int length,int bufsize)
   int cnum;
   int outsize = 0;
   uint16 dptr_num=SVAL(inbuf,smb_vwv0);
-  void *dptr;
 
   cnum = SVAL(inbuf,smb_tid);
 
   DEBUG(3,("reply_findclose, cnum = %d, dptr_num = %d\n", cnum, dptr_num));
 
-  if((dptr = dptr_get(dptr_num))==NULL)
-    {
-      /* Invalid dptr_num */
-      DEBUG(5,("reply_findclose - dptr_num is bad\n"));
-
-      return (ERROR(ERRDOS,ERRbadfid));
-    }
-
-  dptr_demote(dptr_num);
   dptr_close(dptr_num);
 
   outsize = set_message(outbuf,0,0,True);
